@@ -18,7 +18,7 @@ try:
 except ImportError:
     wandb.termwarn("ipython is not supported in python 2.7, upgrade to 3.x")
 
-    class Magics(object):
+    class Magics:
         pass
 
     def magics_class(*args, **kwargs):
@@ -52,7 +52,7 @@ def quiet():
     return False
 
 
-class IFrame(object):
+class IFrame:
     def __init__(self, path=None, opts=None):
         self.path = path
         self.api = wandb.Api()
@@ -89,13 +89,13 @@ class IFrame(object):
                     )
             return object.to_html(self.height, hidden=False)
         except wandb.Error as e:
-            return "Can't display wandb interface<br/>{}".format(e)
+            return f"Can't display wandb interface<br/>{e}"
 
 
 @magics_class
 class WandBMagics(Magics):
     def __init__(self, shell, require_interaction=False):
-        super(WandBMagics, self).__init__(shell)
+        super().__init__(shell)
         self.options = {}
 
     @magic_arguments()
@@ -329,7 +329,7 @@ def attempt_colab_login(app_url):
         return None
 
 
-class Notebook(object):
+class Notebook:
     def __init__(self, settings):
         self.outputs = {}
         self.settings = settings
@@ -358,7 +358,7 @@ class Notebook(object):
         relpath = self.settings._jupyter_path
         if relpath:
             if os.path.exists(relpath):
-                with open(relpath, "r") as json_file:
+                with open(relpath) as json_file:
                     data = json.load(json_file)
                     return data
 
@@ -372,10 +372,18 @@ class Notebook(object):
 
         return
 
-    def save_ipynb(self):
+    def save_ipynb(self) -> bool:
         if not self.settings.save_code:
             logger.info("not saving jupyter notebook")
             return False
+        ret = False
+        try:
+            ret = self._save_ipynb()
+        except Exception as e:
+            logger.info(f"Problem saving notebook: {repr(e)}")
+        return ret
+
+    def _save_ipynb(self) -> bool:
         relpath = self.settings._jupyter_path
         logger.info("looking for notebook: %s", relpath)
         if relpath:
@@ -391,11 +399,18 @@ class Notebook(object):
         # TODO: likely only save if the code has changed
         colab_ipynb = attempt_colab_load_ipynb()
         if colab_ipynb:
-            nb_name = colab_ipynb["metadata"]["colab"]["name"]
+            nb_name = (
+                colab_ipynb.get("metadata", {})
+                .get("colab", {})
+                .get("name", "colab.ipynb")
+            )
             if ".ipynb" not in nb_name:
                 nb_name += ".ipynb"
             with open(
-                os.path.join(self.settings._tmp_code_dir, nb_name,),
+                os.path.join(
+                    self.settings._tmp_code_dir,
+                    nb_name,
+                ),
                 "w",
                 encoding="utf-8",
             ) as f:
