@@ -3,13 +3,19 @@ import os
 import queue
 import tempfile
 import time
+from typing import Optional, Tuple, TYPE_CHECKING
 
 import wandb
+from wandb.filesync import dir_watcher, stats, step_checksum, step_upload
 import wandb.util
 
-from wandb.filesync import stats
-from wandb.filesync import step_checksum
-from wandb.filesync import step_upload
+if TYPE_CHECKING:
+    from wandb.sdk.interface import artifacts
+    from wandb.sdk.internal import (
+        artifacts as internal_artifacts,
+        file_stream,
+        internal_api,
+    )
 
 
 # Temporary directory for copies we make of some file types to
@@ -31,15 +37,20 @@ class FilePusher:
 
     MAX_UPLOAD_JOBS = 64
 
-    def __init__(self, api, file_stream, silent=False):
+    def __init__(
+        self,
+        api: "internal_api.Api",
+        file_stream: "file_stream.FileStreamApi",
+        silent: Optional[bool] = False,
+    ) -> None:
         self._api = api
 
         self._tempdir = tempfile.TemporaryDirectory("wandb")
 
         self._stats = stats.Stats()
 
-        self._incoming_queue = queue.Queue()
-        self._event_queue = queue.Queue()
+        self._incoming_queue: "queue.Queue[step_checksum.Event]" = queue.Queue()
+        self._event_queue: "queue.Queue[step_upload.Event]" = queue.Queue()
 
         self._step_checksum = step_checksum.StepChecksum(
             self._api,
@@ -56,16 +67,16 @@ class FilePusher:
             self._event_queue,
             self.MAX_UPLOAD_JOBS,
             file_stream=file_stream,
-            silent=silent,
+            silent=bool(silent),
         )
         self._step_upload.start()
 
-    def get_status(self):
+    def get_status(self) -> Tuple[bool, stats.Summary]:
         running = self.is_alive()
         summary = self._stats.summary()
         return running, summary
 
-    def print_status(self, prefix=True):
+    def print_status(self, prefix: bool = True) -> None:
         step = 0
         spinner_states = ["-", "\\", "|", "/"]
         stop = False
@@ -74,9 +85,9 @@ class FilePusher:
                 stop = True
             summary = self._stats.summary()
             line = " {:.2f}MB of {:.2f}MB uploaded ({:.2f}MB deduped)\r".format(
-                summary["uploaded_bytes"] / 1048576.0,
-                summary["total_bytes"] / 1048576.0,
-                summary["deduped_bytes"] / 1048576.0,
+                summary.uploaded_bytes / 1048576.0,
+                summary.total_bytes / 1048576.0,
+                summary.deduped_bytes / 1048576.0,
             )
             line = spinner_states[step % 4] + line
             step += 1
@@ -85,8 +96,8 @@ class FilePusher:
                 break
             time.sleep(0.25)
         dedupe_fraction = (
-            summary["deduped_bytes"] / float(summary["total_bytes"])
-            if summary["total_bytes"] > 0
+            summary.deduped_bytes / float(summary.total_bytes)
+            if summary.total_bytes > 0
             else 0
         )
         if dedupe_fraction > 0.01:
@@ -98,18 +109,18 @@ class FilePusher:
         # clear progress line.
         wandb.termlog(" " * 79, prefix=prefix)
 
-    def file_counts_by_category(self):
+    def file_counts_by_category(self) -> stats.FileCountsByCategory:
         return self._stats.file_counts_by_category()
 
     def file_changed(
         self,
-        save_name,
-        path,
-        artifact_id=None,
-        copy=True,
-        use_prepare_flow=False,
-        save_fn=None,
-        digest=None,
+        save_name: dir_watcher.SaveName,
+        path: str,
+        artifact_id: Optional[str] = None,
+        copy: bool = True,
+        use_prepare_flow: bool = False,
+        save_fn: Optional[step_upload.SaveFn] = None,
+        digest: Optional[str] = None,
     ):
         """Tell the file pusher that a file's changed and should be uploaded.
         Arguments:
@@ -123,33 +134,48 @@ class FilePusher:
         if os.path.getsize(path) == 0:
             return
 
-        save_name = wandb.util.to_forward_slash_path(save_name)
+        save_name = dir_watcher.SaveName(wandb.util.to_forward_slash_path(save_name))
         event = step_checksum.RequestUpload(
-            path, save_name, artifact_id, copy, use_prepare_flow, save_fn, digest
+            path,
+            dir_watcher.SaveName(save_name),
+            artifact_id,
+            copy,
+            use_prepare_flow,
+            save_fn,
+            digest,
         )
         self._incoming_queue.put(event)
 
-    def store_manifest_files(self, manifest, artifact_id, save_fn):
+    def store_manifest_files(
+        self,
+        manifest: "artifacts.ArtifactManifest",
+        artifact_id: str,
+        save_fn: "internal_artifacts.SaveFn",
+    ) -> None:
         event = step_checksum.RequestStoreManifestFiles(manifest, artifact_id, save_fn)
         self._incoming_queue.put(event)
 
     def commit_artifact(
-        self, artifact_id, finalize=True, before_commit=None, on_commit=None
+        self,
+        artifact_id: str,
+        finalize: bool = True,
+        before_commit: Optional[step_upload.PreCommitFn] = None,
+        on_commit: Optional[step_upload.PostCommitFn] = None,
     ):
         event = step_checksum.RequestCommitArtifact(
             artifact_id, finalize, before_commit, on_commit
         )
         self._incoming_queue.put(event)
 
-    def finish(self, callback=None):
+    def finish(self, callback: Optional[step_upload.OnRequestFinishFn] = None):
         logger.info("shutting down file pusher")
         self._incoming_queue.put(step_checksum.RequestFinish(callback))
 
-    def join(self):
+    def join(self) -> None:
         # NOTE: must have called finish before join
         logger.info("waiting for file pusher")
         while self.is_alive():
             time.sleep(0.5)
 
-    def is_alive(self):
+    def is_alive(self) -> bool:
         return self._step_checksum.is_alive() or self._step_upload.is_alive()
