@@ -10,7 +10,9 @@ import wandb
 from wandb import util
 from wandb.vendor.pynvml import pynvml
 
+from . import ipu
 from . import tpu
+from .settings_static import SettingsStatic
 from ..interface.interface_queue import InterfaceQueue
 from ..lib import telemetry
 
@@ -25,17 +27,15 @@ StatsDict = Dict[str, Union[float, Dict[str, float]]]
 M1_MAX_POWER_WATTS = 16.5
 
 
-def gpu_in_use_by_this_process(gpu_handle: GPUHandle) -> bool:
+def gpu_in_use_by_this_process(gpu_handle: GPUHandle, pid: int) -> bool:
     if not psutil:
         return False
 
-    # NOTE: this optimizes for the case where wandb was initialized from
-    # iniside the user script (i.e. `wandb.init()`). If we ran using
-    # `wandb run` on the command line, the shell will be detected as the
-    # parent, possible resulting in sibling processes being incorrectly
-    # indentified as part of this process -- still better than not
-    # detecting in-use gpus at all.
-    base_process = psutil.Process().parent() or psutil.Process()
+    try:
+        base_process = psutil.Process(pid=pid)
+    except psutil.NoSuchProcess:
+        # do not report any gpu metrics if the base process cant be found
+        return False
 
     our_processes = base_process.children(recursive=True)
     our_processes.append(base_process)
@@ -62,17 +62,19 @@ class SystemStats:
     _interface: InterfaceQueue
     sampler: SamplerDict
     samples: int
+    _settings: SettingsStatic
     _thread: Optional[threading.Thread]
     gpu_count: int
 
-    def __init__(self, pid: int, interface: InterfaceQueue) -> None:
+    def __init__(self, settings: SettingsStatic, interface: InterfaceQueue) -> None:
         try:
             pynvml.nvmlInit()
             self.gpu_count = pynvml.nvmlDeviceGetCount()
         except pynvml.NVMLError:
             self.gpu_count = 0
         # self.run = run
-        self._pid = pid
+        self._settings = settings
+        self._pid = settings._stats_pid
         self._interface = interface
         self.sampler = {}
         self.samples = 0
@@ -94,6 +96,14 @@ class SystemStats:
             except Exception as e:
                 wandb.termlog("Error initializing TPUProfiler: " + str(e))
 
+        self._ipu_profiler = None
+
+        if ipu.is_ipu_available():
+            try:
+                self._ipu_profiler = ipu.IPUProfiler(self._pid)
+            except Exception as e:
+                wandb.termlog("Error initializing IPUProfiler: " + str(e))
+
     def start(self) -> None:
         if self._thread is None:
             self._shutdown = False
@@ -112,14 +122,16 @@ class SystemStats:
     @property
     def sample_rate_seconds(self) -> float:
         """Sample system stats every this many seconds, defaults to 2, min is 0.5"""
-        return 2
-        # return max(0.5, self._api.dynamic_settings["system_sample_seconds"])
+        sample_rate = self._settings._stats_sample_rate_seconds
+        # TODO: handle self._api.dynamic_settings["system_sample_seconds"]
+        return max(0.5, sample_rate)
 
     @property
     def samples_to_average(self) -> int:
         """The number of samples to average before pushing, defaults to 15 valid range (2:30)"""
-        return 15
-        # return min(30, max(2, self._api.dynamic_settings["system_samples"]))
+        samples = self._settings._stats_samples_to_average
+        # TODO: handle self._api.dynamic_settings["system_samples"]
+        return min(30, max(2, samples))
 
     def _thread_body(self) -> None:
         while True:
@@ -176,7 +188,7 @@ class SystemStats:
                 temp = pynvml.nvmlDeviceGetTemperature(
                     handle, pynvml.NVML_TEMPERATURE_GPU
                 )
-                in_use_by_us = gpu_in_use_by_this_process(handle)
+                in_use_by_us = gpu_in_use_by_this_process(handle, pid=self._pid)
 
                 stats["gpu.{}.{}".format(i, "gpu")] = utilz.gpu
                 stats["gpu.{}.{}".format(i, "memory")] = utilz.memory
@@ -267,4 +279,7 @@ class SystemStats:
             tpu_utilization = self._tpu_profiler.get_tpu_utilization()
             if tpu_utilization is not None:
                 stats["tpu"] = tpu_utilization
+
+        if self._ipu_profiler:
+            stats.update(self._ipu_profiler.get_metrics())
         return stats
