@@ -30,9 +30,10 @@ class WBURLs:
             doc_run="https://wandb.me/run",
             doc_require="https://wandb.me/library-require",
             doc_start_err="https://docs.wandb.ai/library/init#init-start-error",
-            upgrade_local="https://wandb.me/local-upgrade",
+            upgrade_server="https://wandb.me/server-upgrade",
             multiprocess="http://wandb.me/init-multiprocess",
             wandb_init="https://wandb.me/wandb-init",
+            wandb_server="https://wandb.me/wandb-server",
         )
 
     def get(self, s: "URLS") -> str:
